#pragma once

#include <fstream>
#include <vector>
#include <algorithm>
#include <iostream>
#include <unordered_set>
#include <ranges>
#include <cmath>
#include <functional>
#include <cmath>

#include "GeometryStructures/Node.h"
#include "GeometryStructures/FiniteElement.h"
#include "GeometryStructures/BoundaryFiniteElement.h"
#include "GeometryStructures/Edge.h"

float scalarMultiply(const std::vector<float> &p1, const std::vector<float> &p2);

float distance(const std::vector<float> &p1, const std::vector<float> &p2);

float angle(const std::vector<float> &p, const std::vector<std::vector<float>> &otherPoints);
class MeshLoader {

 public:
  virtual void LoadMesh(const std::string &) = 0;
  virtual ~MeshLoader() = default;

  std::vector<Node> &GetNodes();
  std::vector<FiniteElement> &GetFiniteElements();
  std::vector<BoundaryFiniteElement> &GetBoundaryFiniteElements();

  std::vector<int> GetFiniteElementsByNodesId(int id1, int id2, int id3);
  std::vector<int> GetFiniteElementsByEdge(int firstVerticeId, int secondVerticeId);
  std::unordered_set<int> GetVertexNodesBySurfaceId(int surfaceId);

  std::vector<int> GetFiniteElementsByMaterialId(int materialId);
  std::vector<int> GetBboundaryFiniteElementsBySurfaceId(int id);

  void InsertMiddle();

  std::vector<std::unordered_set<int>> GetNodesNeighbors();

  void SortByMinAngle();
 private:
  float minAngle(const BoundaryFiniteElement &element);
  Node createMiddleNode(const Edge &edge);
 protected:
  std::vector<Node> nodes;
  std::vector<FiniteElement> finiteElements;
  std::vector<BoundaryFiniteElement> boundaryFiniteElements;
 private:
};
